@@ -28,21 +28,20 @@
  * service as multiple SSDP resources (UPnP does this for example).
  */
 
+#ifdef HAVE_CONFIG_H
 #include <config.h>
-#include <string.h>
-#include <stdlib.h>
-#include <time.h>
-
-#include <libsoup/soup.h>
+#endif /* HAVE_CONFIG_H */
 
 #include "gssdp-resource-group.h"
 #include "gssdp-resource-browser.h"
 #include "gssdp-client-private.h"
 #include "gssdp-protocol.h"
 
-G_DEFINE_TYPE (GSSDPResourceGroup,
-               gssdp_resource_group,
-               G_TYPE_OBJECT);
+#include <string.h>
+#include <stdlib.h>
+#include <time.h>
+
+#include <libsoup/soup.h>
 
 #define DEFAULT_MAN_HEADER "\"ssdp:discover\""
 
@@ -65,6 +64,11 @@ struct _GSSDPResourceGroupPrivate {
         GQueue      *message_queue;
         GSource     *message_src;
 };
+typedef struct _GSSDPResourceGroupPrivate GSSDPResourceGroupPrivate;
+
+G_DEFINE_TYPE_WITH_PRIVATE (GSSDPResourceGroup,
+                            gssdp_resource_group,
+                            G_TYPE_OBJECT);
 
 enum {
         PROP_0,
@@ -141,15 +145,14 @@ send_initial_resource_byebye    (Resource          *resource);
 static void
 gssdp_resource_group_init (GSSDPResourceGroup *resource_group)
 {
-        resource_group->priv = G_TYPE_INSTANCE_GET_PRIVATE
-                                        (resource_group,
-                                         GSSDP_TYPE_RESOURCE_GROUP,
-                                         GSSDPResourceGroupPrivate);
+        GSSDPResourceGroupPrivate *priv;
 
-        resource_group->priv->max_age = SSDP_DEFAULT_MAX_AGE;
-        resource_group->priv->message_delay = DEFAULT_MESSAGE_DELAY;
+        priv = gssdp_resource_group_get_instance_private (resource_group);
 
-        resource_group->priv->message_queue = g_queue_new ();
+        priv->max_age = SSDP_DEFAULT_MAX_AGE;
+        priv->message_delay = DEFAULT_MESSAGE_DELAY;
+
+        priv->message_queue = g_queue_new ();
 }
 
 static void
@@ -230,7 +233,7 @@ gssdp_resource_group_dispose (GObject *object)
         GSSDPResourceGroupPrivate *priv;
 
         resource_group = GSSDP_RESOURCE_GROUP (object);
-        priv = resource_group->priv;
+        priv = gssdp_resource_group_get_instance_private (resource_group);
 
         g_list_free_full (priv->resources, (GFreeFunc) resource_free);
         priv->resources = NULL;
@@ -245,19 +248,13 @@ gssdp_resource_group_dispose (GObject *object)
                                         (priv->message_queue));
                 }
 
-                g_queue_free (priv->message_queue);
-                priv->message_queue = NULL;
+                g_clear_pointer (&priv->message_queue, g_queue_free);
         }
 
-        if (priv->message_src) {
-                g_source_destroy (priv->message_src);
-                priv->message_src = NULL;
-        }
 
-        if (priv->timeout_src) {
-                g_source_destroy (priv->timeout_src);
-                priv->timeout_src = NULL;
-        }
+        /* No need to unref sources, already done on creation */
+        g_clear_pointer (&priv->message_src, g_source_destroy);
+        g_clear_pointer (&priv->timeout_src, g_source_destroy);
 
         if (priv->client) {
                 if (g_signal_handler_is_connected
@@ -267,9 +264,8 @@ gssdp_resource_group_dispose (GObject *object)
                                 (priv->client,
                                  priv->message_received_id);
                 }
-                                                   
-                g_object_unref (priv->client);
-                priv->client = NULL;
+
+                g_clear_object (&priv->client);
         }
 
         G_OBJECT_CLASS (gssdp_resource_group_parent_class)->dispose (object);
@@ -286,8 +282,6 @@ gssdp_resource_group_class_init (GSSDPResourceGroupClass *klass)
         object_class->get_property = gssdp_resource_group_get_property;
         object_class->dispose      = gssdp_resource_group_dispose;
 
-        g_type_class_add_private (klass, sizeof (GSSDPResourceGroupPrivate));
-
         /**
          * GSSDPResourceGroup:client:
          *
@@ -385,13 +379,16 @@ static void
 gssdp_resource_group_set_client (GSSDPResourceGroup *resource_group,
                                  GSSDPClient        *client)
 {
+        GSSDPResourceGroupPrivate *priv;
+
         g_return_if_fail (GSSDP_IS_RESOURCE_GROUP (resource_group));
         g_return_if_fail (GSSDP_IS_CLIENT (client));
 
-        resource_group->priv->client = g_object_ref (client);
+        priv = gssdp_resource_group_get_instance_private (resource_group);
+        priv->client = g_object_ref (client);
 
-        resource_group->priv->message_received_id =
-                g_signal_connect_object (resource_group->priv->client,
+        priv->message_received_id =
+                g_signal_connect_object (priv->client,
                                          "message-received",
                                          G_CALLBACK (message_received_cb),
                                          resource_group,
@@ -409,9 +406,13 @@ gssdp_resource_group_set_client (GSSDPResourceGroup *resource_group,
 GSSDPClient *
 gssdp_resource_group_get_client (GSSDPResourceGroup *resource_group)
 {
+        GSSDPResourceGroupPrivate *priv;
+
         g_return_val_if_fail (GSSDP_IS_RESOURCE_GROUP (resource_group), NULL);
 
-        return resource_group->priv->client;
+        priv = gssdp_resource_group_get_instance_private (resource_group);
+
+        return priv->client;
 }
 
 /**
@@ -425,13 +426,16 @@ void
 gssdp_resource_group_set_max_age (GSSDPResourceGroup *resource_group,
                                   guint               max_age)
 {
+        GSSDPResourceGroupPrivate *priv;
+
         g_return_if_fail (GSSDP_IS_RESOURCE_GROUP (resource_group));
 
-        if (resource_group->priv->max_age == max_age)
+        priv = gssdp_resource_group_get_instance_private (resource_group);
+        if (priv->max_age == max_age)
                 return;
 
-        resource_group->priv->max_age = max_age;
-        
+        priv->max_age = max_age;
+
         g_object_notify (G_OBJECT (resource_group), "max-age");
 }
 
@@ -444,9 +448,12 @@ gssdp_resource_group_set_max_age (GSSDPResourceGroup *resource_group,
 guint
 gssdp_resource_group_get_max_age (GSSDPResourceGroup *resource_group)
 {
-        g_return_val_if_fail (GSSDP_IS_RESOURCE_GROUP (resource_group), 0);
+        GSSDPResourceGroupPrivate *priv;
 
-        return resource_group->priv->max_age;
+        g_return_val_if_fail (GSSDP_IS_RESOURCE_GROUP (resource_group), 0);
+        priv = gssdp_resource_group_get_instance_private (resource_group);
+
+        return priv->max_age;
 }
 
 /**
@@ -460,13 +467,16 @@ void
 gssdp_resource_group_set_message_delay (GSSDPResourceGroup *resource_group,
                                         guint               message_delay)
 {
+        GSSDPResourceGroupPrivate *priv;
+
         g_return_if_fail (GSSDP_IS_RESOURCE_GROUP (resource_group));
 
-        if (resource_group->priv->message_delay == message_delay)
+        priv = gssdp_resource_group_get_instance_private (resource_group);
+        if (priv->message_delay == message_delay)
                 return;
 
-        resource_group->priv->message_delay = message_delay;
-        
+        priv->message_delay = message_delay;
+
         g_object_notify (G_OBJECT (resource_group), "message-delay");
 }
 
@@ -479,9 +489,12 @@ gssdp_resource_group_set_message_delay (GSSDPResourceGroup *resource_group,
 guint
 gssdp_resource_group_get_message_delay (GSSDPResourceGroup *resource_group)
 {
-        g_return_val_if_fail (GSSDP_IS_RESOURCE_GROUP (resource_group), 0);
+        GSSDPResourceGroupPrivate *priv;
 
-        return resource_group->priv->message_delay;
+        g_return_val_if_fail (GSSDP_IS_RESOURCE_GROUP (resource_group), 0);
+        priv = gssdp_resource_group_get_instance_private (resource_group);
+
+        return priv->message_delay;
 }
 
 static void
@@ -521,12 +534,15 @@ void
 gssdp_resource_group_set_available (GSSDPResourceGroup *resource_group,
                                     gboolean            available)
 {
+        GSSDPResourceGroupPrivate *priv;
+
         g_return_if_fail (GSSDP_IS_RESOURCE_GROUP (resource_group));
 
-        if (resource_group->priv->available == available)
+        priv = gssdp_resource_group_get_instance_private (resource_group);
+        if (priv->available == available)
                 return;
 
-        resource_group->priv->available = available;
+        priv->available = available;
 
         if (available) {
                 int timeout;
@@ -537,39 +553,38 @@ gssdp_resource_group_set_available (GSSDPResourceGroup *resource_group,
                  * Read the paragraphs about 'CACHE-CONTROL' on pages 21-22 of
                  * UPnP Device Architecture Document v1.1 for further details.
                  * */
-                timeout = resource_group->priv->max_age;
+                timeout = priv->max_age;
                 if (G_LIKELY (timeout > 6))
                         timeout = (timeout / 3) - 1;
 
                 /* Add re-announcement timer */
-                resource_group->priv->timeout_src =
-                        g_timeout_source_new_seconds (timeout);
-                g_source_set_callback (resource_group->priv->timeout_src,
+                priv->timeout_src = g_timeout_source_new_seconds (timeout);
+                g_source_set_callback (priv->timeout_src,
                                        resource_group_timeout,
                                        resource_group, NULL);
 
-                g_source_attach (resource_group->priv->timeout_src,
+                g_source_attach (priv->timeout_src,
                                  g_main_context_get_thread_default ());
 
-                g_source_unref (resource_group->priv->timeout_src);
+                g_source_unref (priv->timeout_src);
 
                 /* Make sure initial byebyes are sent grouped before initial
                  * alives */
-                send_announcement_set (resource_group->priv->resources,
+                send_announcement_set (priv->resources,
                                        (GFunc) send_initial_resource_byebye);
 
-                send_announcement_set (resource_group->priv->resources,
+                send_announcement_set (priv->resources,
                                        (GFunc) resource_alive);
         } else {
                 /* Unannounce all resources */
-                send_announcement_set (resource_group->priv->resources,
+                send_announcement_set (priv->resources,
                                        (GFunc) resource_byebye);
 
                 /* Remove re-announcement timer */
-                g_source_destroy (resource_group->priv->timeout_src);
-                resource_group->priv->timeout_src = NULL;
+                g_source_destroy (priv->timeout_src);
+                priv->timeout_src = NULL;
         }
-        
+
         g_object_notify (G_OBJECT (resource_group), "available");
 }
 
@@ -582,9 +597,12 @@ gssdp_resource_group_set_available (GSSDPResourceGroup *resource_group,
 gboolean
 gssdp_resource_group_get_available (GSSDPResourceGroup *resource_group)
 {
-        g_return_val_if_fail (GSSDP_IS_RESOURCE_GROUP (resource_group), FALSE);
+        GSSDPResourceGroupPrivate *priv;
 
-        return resource_group->priv->available;
+        g_return_val_if_fail (GSSDP_IS_RESOURCE_GROUP (resource_group), FALSE);
+        priv = gssdp_resource_group_get_instance_private (resource_group);
+
+        return priv->available;
 }
 
 /**
@@ -605,15 +623,17 @@ gssdp_resource_group_add_resource (GSSDPResourceGroup *resource_group,
                                    const char         *usn,
                                    GList              *locations)
 {
-        Resource *resource;
-        GList *l;
-        GError *error;
+        GSSDPResourceGroupPrivate *priv = NULL;
+        Resource *resource = NULL;
+        GError *error = NULL;
 
         g_return_val_if_fail (GSSDP_IS_RESOURCE_GROUP (resource_group), 0);
         g_return_val_if_fail (target != NULL, 0);
         g_return_val_if_fail (usn != NULL, 0);
         g_return_val_if_fail (locations != NULL, 0);
 
+        priv = gssdp_resource_group_get_instance_private (resource_group);
+
         resource = g_slice_new0 (Resource);
 
         resource->resource_group = resource_group;
@@ -621,8 +641,9 @@ gssdp_resource_group_add_resource (GSSDPResourceGroup *resource_group,
         resource->target = g_strdup (target);
         resource->usn    = g_strdup (usn);
 
-        error = NULL;
-        resource->target_regex = create_target_regex (target, &resource->version, &error);
+        resource->target_regex = create_target_regex (target,
+                                                      &resource->version,
+                                                      &error);
         if (error) {
                 g_warning ("Error compiling regular expression for '%s': %s",
                            target,
@@ -636,17 +657,13 @@ gssdp_resource_group_add_resource (GSSDPResourceGroup *resource_group,
 
         resource->initial_byebye_sent = FALSE;
 
-        for (l = locations; l; l = l->next) {
-                resource->locations = g_list_append (resource->locations,
-                                                    g_strdup (l->data));
-        }
+        resource->locations = g_list_copy_deep (locations, (GCopyFunc) g_strdup, NULL);
 
-        resource_group->priv->resources =
-                g_list_prepend (resource_group->priv->resources, resource);
+        priv->resources = g_list_prepend (priv->resources, resource);
 
-        resource->id = ++resource_group->priv->last_resource_id;
+        resource->id = ++priv->last_resource_id;
 
-        if (resource_group->priv->available)
+        if (priv->available)
                 resource_alive (resource);
 
         return resource->id;
@@ -674,7 +691,10 @@ gssdp_resource_group_add_resource_simple (GSSDPResourceGroup *resource_group,
         guint  resource_id;
 
         locations = g_list_append (locations, (gpointer) location);
-        resource_id = gssdp_resource_group_add_resource (resource_group, target, usn, locations);
+        resource_id = gssdp_resource_group_add_resource (resource_group,
+                                                         target,
+                                                         usn,
+                                                         locations);
 
         g_list_free (locations);
 
@@ -692,21 +712,22 @@ void
 gssdp_resource_group_remove_resource (GSSDPResourceGroup *resource_group,
                                       guint               resource_id)
 {
+        GSSDPResourceGroupPrivate *priv;
         GList *l;
 
         g_return_if_fail (GSSDP_IS_RESOURCE_GROUP (resource_group));
         g_return_if_fail (resource_id > 0);
 
-        for (l = resource_group->priv->resources; l; l = l->next) {
+        priv = gssdp_resource_group_get_instance_private (resource_group);
+        for (l = priv->resources; l; l = l->next) {
                 Resource *resource;
 
                 resource = l->data;
 
                 if (resource->id == resource_id) {
-                        resource_group->priv->resources = 
-                                g_list_remove (resource_group->priv->resources,
-                                               resource);
-                        
+                        priv->resources = g_list_remove (priv->resources,
+                                                         resource);
+
                         resource_free (resource);
 
                         return;
@@ -721,11 +742,12 @@ static gboolean
 resource_group_timeout (gpointer user_data)
 {
         GSSDPResourceGroup *resource_group;
+        GSSDPResourceGroupPrivate *priv;
 
         resource_group = GSSDP_RESOURCE_GROUP (user_data);
+        priv = gssdp_resource_group_get_instance_private (resource_group);
 
-        send_announcement_set (resource_group->priv->resources,
-                               (GFunc) resource_alive);
+        send_announcement_set (priv->resources, (GFunc) resource_alive);
 
         return TRUE;
 }
@@ -742,15 +764,17 @@ message_received_cb (G_GNUC_UNUSED GSSDPClient *client,
                      gpointer                   user_data)
 {
         GSSDPResourceGroup *resource_group;
+        GSSDPResourceGroupPrivate *priv;
         const char *target, *mx_str, *version_str, *man;
         gboolean want_all;
         int mx, version;
         GList *l;
 
         resource_group = GSSDP_RESOURCE_GROUP (user_data);
+        priv = gssdp_resource_group_get_instance_private (resource_group);
 
         /* Only process if we are available */
-        if (!resource_group->priv->available)
+        if (!priv->available)
                 return;
 
         /* We only handle discovery requests */
@@ -759,7 +783,7 @@ message_received_cb (G_GNUC_UNUSED GSSDPClient *client,
 
         /* Extract target */
         target = soup_message_headers_get_one (headers, "ST");
-        if (!target) {
+        if (target == NULL) {
                 g_warning ("Discovery request did not have an ST header");
 
                 return;
@@ -770,14 +794,14 @@ message_received_cb (G_GNUC_UNUSED GSSDPClient *client,
 
         /* Extract MX */
         mx_str = soup_message_headers_get_one (headers, "MX");
-        if (!mx_str || atoi (mx_str) <= 0) {
+        if (mx_str == NULL || atoi (mx_str) <= 0) {
                 g_warning ("Discovery request did not have a valid MX header");
 
                 return;
         }
 
         man = soup_message_headers_get_one (headers, "MAN");
-        if (!man || strcmp (man, DEFAULT_MAN_HEADER) != 0) {
+        if (man == NULL || strcmp (man, DEFAULT_MAN_HEADER) != 0) {
                 g_warning ("Discovery request did not have a valid MAN header");
 
                 return;
@@ -793,7 +817,7 @@ message_received_cb (G_GNUC_UNUSED GSSDPClient *client,
                 version = 0;
 
         /* Find matching resource */
-        for (l = resource_group->priv->resources; l; l = l->next) {
+        for (l = priv->resources; l != NULL; l = l->next) {
                 Resource *resource;
 
                 resource = l->data;
@@ -813,7 +837,7 @@ message_received_cb (G_GNUC_UNUSED GSSDPClient *client,
 
                         /* Prepare response */
                         response = g_slice_new (DiscoveryResponse);
-                        
+
                         response->dest_ip   = g_strdup (from_ip);
                         response->dest_port = from_port;
                         response->resource  = resource;
@@ -833,7 +857,7 @@ message_received_cb (G_GNUC_UNUSED GSSDPClient *client,
                                          g_main_context_get_thread_default ());
 
                         g_source_unref (response->timeout_src);
-                        
+
                         /* Add to resource */
                         resource->responses =
                                 g_list_prepend (resource->responses, response);
@@ -847,23 +871,24 @@ message_received_cb (G_GNUC_UNUSED GSSDPClient *client,
 static char *
 construct_al (Resource *resource)
 {
-       if (resource->locations->next) {
-                GString *al_string;
-                GList *l;
+        GString *al_string;
+        GList *l;
 
-                al_string = g_string_new ("AL: ");
+        if (resource->locations->next == NULL) {
+                return NULL;
+        }
 
-                for (l = resource->locations->next; l; l = l->next) {
-                        g_string_append_c (al_string, '<');
-                        g_string_append (al_string, l->data);
-                        g_string_append_c (al_string, '>');
-                }
+        al_string = g_string_new ("AL: ");
 
-                g_string_append (al_string, "\r\n");
+        for (l = resource->locations->next; l; l = l->next) {
+                g_string_append_c (al_string, '<');
+                g_string_append (al_string, l->data);
+                g_string_append_c (al_string, '>');
+        }
 
-                return g_string_free (al_string, FALSE);
-        } else
-                return NULL; 
+        g_string_append (al_string, "\r\n");
+
+        return g_string_free (al_string, FALSE);
 }
 
 static char *
@@ -893,19 +918,21 @@ construct_usn (const char *usn,
 static gboolean
 discovery_response_timeout (gpointer user_data)
 {
-        DiscoveryResponse *response;
+        DiscoveryResponse *response = user_data;
         GSSDPClient *client;
         SoupDate *date;
         char *al, *date_str, *message;
         guint max_age;
         char *usn;
+        GSSDPResourceGroup *self = response->resource->resource_group;
+        GSSDPResourceGroupPrivate *priv;
 
-        response = user_data;
+        priv = gssdp_resource_group_get_instance_private (self);
 
         /* Send message */
-        client = response->resource->resource_group->priv->client;
+        client = priv->client;
 
-        max_age = response->resource->resource_group->priv->max_age;
+        max_age = priv->max_age;
 
         al = construct_al (response->resource);
         usn = construct_usn (response->resource->usn,
@@ -950,7 +977,7 @@ discovery_response_free (DiscoveryResponse *response)
                 g_list_remove (response->resource->responses, response);
 
         g_source_destroy (response->timeout_src);
-        
+
         g_free (response->dest_ip);
         g_free (response->target);
 
@@ -964,31 +991,31 @@ static gboolean
 process_queue (gpointer data)
 {
         GSSDPResourceGroup *resource_group;
+        GSSDPResourceGroupPrivate *priv;
+        GSSDPClient *client;
+        char *message;
 
         resource_group = GSSDP_RESOURCE_GROUP (data);
+        priv = gssdp_resource_group_get_instance_private (resource_group);
 
-        if (g_queue_is_empty (resource_group->priv->message_queue)) {
+        if (g_queue_is_empty (priv->message_queue)) {
                 /* this is the timeout after last message in queue */
-                resource_group->priv->message_src = NULL;
+                priv->message_src = NULL;
 
                 return FALSE;
-        } else {
-                GSSDPClient *client;
-                char *message;
-
-                client = resource_group->priv->client;
-                message = g_queue_pop_head
-                        (resource_group->priv->message_queue);
-
-                _gssdp_client_send_message (client,
-                                            NULL,
-                                            0,
-                                            message,
-                                            _GSSDP_DISCOVERY_RESPONSE);
-                g_free (message);
-
-                return TRUE;
         }
+
+        client = priv->client;
+        message = g_queue_pop_head (priv->message_queue);
+
+        _gssdp_client_send_message (client,
+                                    NULL,
+                                    0,
+                                    message,
+                                    _GSSDP_DISCOVERY_RESPONSE);
+        g_free (message);
+
+        return TRUE;
 }
 
 /*
@@ -1000,21 +1027,26 @@ static void
 queue_message (GSSDPResourceGroup *resource_group,
                char               *message)
 {
-        g_queue_push_tail (resource_group->priv->message_queue, 
-                           message);
+        GSSDPResourceGroupPrivate *priv;
+        priv = gssdp_resource_group_get_instance_private (resource_group);
 
-        if (resource_group->priv->message_src == NULL) {
-                /* nothing in the queue: process message immediately 
-                   and add a timeout for (possible) next message */
-                process_queue (resource_group);
-                resource_group->priv->message_src = g_timeout_source_new (
-                    resource_group->priv->message_delay);
-                g_source_set_callback (resource_group->priv->message_src,
-                    process_queue, resource_group, NULL);
-                g_source_attach (resource_group->priv->message_src,
-                                 g_main_context_get_thread_default ());
-                g_source_unref (resource_group->priv->message_src);
+        g_queue_push_tail (priv->message_queue, message);
+
+        if (priv->message_src != NULL) {
+                return;
         }
+
+        /* nothing in the queue: process message immediately
+           and add a timeout for (possible) next message */
+        process_queue (resource_group);
+        priv->message_src = g_timeout_source_new (priv->message_delay);
+        g_source_set_callback (priv->message_src,
+                               process_queue,
+                               resource_group,
+                               NULL);
+        g_source_attach (priv->message_src,
+                        g_main_context_get_thread_default ());
+        g_source_unref (priv->message_src);
 }
 
 /*
@@ -1023,21 +1055,35 @@ queue_message (GSSDPResourceGroup *resource_group,
 static void
 resource_alive (Resource *resource)
 {
+        GSSDPResourceGroupPrivate *priv;
         GSSDPClient *client;
         guint max_age;
         char *al, *message;
+        const char *group;
+        char *dest;
+
+        priv = gssdp_resource_group_get_instance_private
+                                        (resource->resource_group);
 
         /* Send initial byebye if not sent already */
         send_initial_resource_byebye (resource);
 
         /* Send message */
-        client = resource->resource_group->priv->client;
+        client = priv->client;
 
-        max_age = resource->resource_group->priv->max_age;
+        max_age = priv->max_age;
 
         al = construct_al (resource);
 
+        /* FIXME: UGLY V6 stuff */
+        group = _gssdp_client_get_mcast_group (client);
+        if (strchr (group, ':') != NULL)
+                dest = g_strdup_printf ("[%s]", group);
+        else
+                dest = g_strdup (group);
+
         message = g_strdup_printf (SSDP_ALIVE_MESSAGE,
+                                   dest,
                                    max_age,
                                    (char *) resource->locations->data,
                                    al ? al : "",
@@ -1047,6 +1093,7 @@ resource_alive (Resource *resource)
 
         queue_message (resource->resource_group, message);
 
+        g_free (dest);
         g_free (al);
 }
 
@@ -1056,14 +1103,31 @@ resource_alive (Resource *resource)
 static void
 resource_byebye (Resource *resource)
 {
-        char *message;
+        char *message = NULL;
+        const char *group = NULL;
+        char *host = NULL;
+        GSSDPResourceGroupPrivate *priv = NULL;
+        GSSDPClient *client = NULL;
+
+        priv = gssdp_resource_group_get_instance_private (resource->resource_group);
+        client = priv->client;
+
+        /* FIXME: UGLY V6 stuff */
+        group = _gssdp_client_get_mcast_group (client);
+        if (strchr (group, ':') != NULL)
+                host = g_strdup_printf ("[%s]", group);
+        else
+                host = g_strdup (group);
 
         /* Queue message */
         message = g_strdup_printf (SSDP_BYEBYE_MESSAGE,
+                                   host,
                                    resource->target,
                                    resource->usn);
 
         queue_message (resource->resource_group, message);
+
+        g_free (host);
 }
 
 /*
@@ -1072,23 +1136,23 @@ resource_byebye (Resource *resource)
 static void
 resource_free (Resource *resource)
 {
+        GSSDPResourceGroupPrivate *priv;
+
+        priv = gssdp_resource_group_get_instance_private
+                                        (resource->resource_group);
+        /* discovery_response_free will take clear of freeing the list
+         * elements and data */
         while (resource->responses)
                 discovery_response_free (resource->responses->data);
 
-        if (resource->resource_group->priv->available)
+        if (priv->available)
                 resource_byebye (resource);
 
         g_free (resource->usn);
         g_free (resource->target);
 
-        if (resource->target_regex)
-                g_regex_unref (resource->target_regex);
-
-        while (resource->locations) {
-                g_free (resource->locations->data);
-                resource->locations = g_list_delete_link (resource->locations,
-                                                         resource->locations);
-        }
+        g_clear_pointer (&resource->target_regex, g_regex_unref);
+        g_list_free_full (resource->locations, g_free);
 
         g_slice_free (Resource, resource);
 }
@@ -1126,6 +1190,9 @@ create_target_regex (const char *target, guint *version, GError **error)
 
         version_str = get_version_for_target (pattern);
         if (version_str != NULL) {
+                /* version_str is now at : + 1 in the pattern string, so this
+                 * replaces the actual version with the number matching regex
+                 */
                 *version = atoi (version_str);
                 strcpy (version_str, VERSION_PATTERN);
         }
