@@ -19,14 +19,15 @@
  * Boston, MA 02110-1301, USA.
  */
 
-#ifndef __GSSDP_SOCKET_FUNCTIONS_H
-#define __GSSDP_SOCKET_FUNCTIONS_H
+#ifndef GSSDP_SOCKET_FUNCTIONS_H
+#define GSSDP_SOCKET_FUNCTIONS_H
 
 #include <gio/gio.h>
 
 G_GNUC_INTERNAL gboolean
 gssdp_socket_mcast_interface_set (GSocket       *socket,
                                   GInetAddress  *iface_address,
+                                  gint           index,
                                   GError       **error);
 G_GNUC_INTERNAL gboolean
 gssdp_socket_reuse_address       (GSocket *socket,
