@@ -89,7 +89,7 @@ clear_packet_treeview (void)
         while (more) {
                 gtk_tree_model_get (model,
                                 &iter, 
-                                5, &arrival_time, -1);
+                                6, &arrival_time, -1);
                 g_free (arrival_time);
                 more = gtk_list_store_remove (GTK_LIST_STORE (model), &iter);
         }
@@ -748,7 +748,7 @@ init_upnp (void)
         client = g_initable_new (GSSDP_TYPE_CLIENT,
                                  NULL,
                                  &error,
-                                 "host-ip", prefer_v6 ? "::" : NULL,
+                                 "address-family", prefer_v6 ? G_SOCKET_FAMILY_IPV6 : G_SOCKET_FAMILY_IPV4,
                                  "interface", interface,
                                  NULL);
         if (error) {
